from .actions import (
    do,
    translate,
    get_simple_temp,
    get_now_temp,
    easy_ocr,
    simple_translate,
    meme_gen,
    web_screenshot,
    create_folder,
    create,
    find,
    list_all,
)

# from .actions import *
from .base import start



import time
import copy
import typing as t
import schedule as sche
from functools import wraps
from .utils import convert_time, callable_check, drop


class Action:

    # procs: t.Sequence[t.Callable]
    procs: t.Union[t.List, "ProcManager"]

    def once(self):
        ret = self._excute()
        return ret

    def every(self, freq) -> None:
        # shoud be in convert_time ?
        if isinstance(freq, dict):
            freq = convert_time(freq)

        sche.every(freq).seconds.do(self.once)

    def when(self, time: str):
        sche.every().day.at(time).do(self.once)

    def _excute(self):
        if not self.procs:
            raise Exception()
        elif isinstance(self.procs, list):
            self.procs = ProcManager(*self.procs)

        # seed = copy.deepcopy(self.__dict__)
        seed = self.__dict__.copy()
        seed = drop(seed, ["procs", "self"])
        # Wait... there is no self in self.__dict__
        # Seems like self should be when @action decorated a method
        # little dangerous here drop self only use key-name,
        # think about it, if I use 'this' instead of 'self' wow...
        ret = self.procs.excute(seed)
        # ret = self.procs(seed)
        # ret = self.procs(self.__dict__)
        # DONE Need change procs() -> procs.excute() -> way better
        return ret


class ProcManager:
    def __init__(self, *procs) -> None:
        map(callable_check, procs)
        # self._procs += procs -> Because instances share their class _proc
        self._procs = procs

    def __iter__(self) -> t.Callable:
        for f in self._procs:
            yield f

    def __add__(self, other):
        _new_procs = self._procs + other._procs
        return ProcManager(*_new_procs)

    def __call__(self, seed):
        ret = self.pipe(seed, *self._procs)
        return ret

    def excute(self, seed):
        ret = self.pipe(seed, *self._procs)
        return ret

    def pipe(self, seed, *funcs) -> t.Any:

        for func in funcs:
            if isinstance(seed, dict):
                seed = func(**seed)
                continue
            seed = func(seed)
        return seed

    # @staticmethod
    # def pipe(seed, *funcs)->t.Any:
    #     for func in funcs:
    #         if isinstance(seed, dict):
    #             seed = func(**seed)
    #             continue
    #         seed = func(seed)
    #     return seed


class action(Action):
    def __init__(self, procs: t.List) -> None:
        self.procs = procs

    def __call__(self, decorated):
        # TODO really need a smarter way to detect self, and get result from procs
        @wraps(decorated)
        def instead(*args, **kwd):
            seed = decorated(*args, **kwd)  # when decorate a method, self in args...
            self.__dict__.update(seed)
            return self

        return instead


def require():
    pass


def start():
    while True:
        time.sleep(1)
        sche.run_pending()



import codecs
from setuptools import setup
from pathlib import Path

URGE_VERSION = "0.4.5.1"
DOWNLOAD_URL = ""


def get_packages(package):
    """
    Return root package and all sub-packages.
    """
    return [str(path.parent) for path in Path(package).glob("**/__init__.py")]


def read_file(filename):
    """
    Read a utf8 encoded text file and return its contents.
    """
    with codecs.open(filename, "r", "utf8") as f:
        return f.read()


setup(
    name="urge",
    packages=get_packages("urge"),
    version=URGE_VERSION,
    description="You've got the Urge to write some code ,so go ahead",
    long_description=read_file("README.md"),
    license="MIT",
    author="Hou",
    author_email="hhhoujue@gmail.com",
    url="",
    download_url=DOWNLOAD_URL,
    keywords=["schedule", "jobs", "toy", "file", "utils"],
    install_requires=[
        "playwright==1.13.1",
        "schedule==1.1.0",
        "pyquery",
        "daoism",
        "httpx",
        "path",
        "fs",
    ],
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
        "Programming Language :: Python",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.6",
        "Programming Language :: Python :: 3.7",
        "Programming Language :: Python :: 3.8",
        "Programming Language :: Python :: 3.9",
        "Natural Language :: English",
    ],
    python_requires=">=3.7",
)



def web_screenshot(url: str, **kwargs) -> str:
    from playwright.sync_api import sync_playwright
    from datetime import datetime

    print("🤖 : Chromium import is OK...")

    # add a new line
    # p = sync_playwright().start()
    # iphone_11 = p.devices["iPhone 11 Pro"]
    # browser_type = p.chromium
    # browser = browser_type.launch(headless=True)

    # print(
    #     '🤖 : Launching browser... Please wait for a while then you will see the result.'
    # )
    # context = browser.new_context(**iphone_11)
    # page = context.new_page()
    # page.set_viewport_size({"width": 375, "height": 635})
    # page.goto(url, wait_until="networkidle")
    # file_name = f'screenshot{datetime.now().strftime("%m-%d_%H:%M:%S")}.png'
    # page.screenshot(path=file_name)
    # page.close()
    # browser.close()

    with sync_playwright() as p:
        iphone_11 = p.devices["iPhone 11 Pro"]
        browser_type = p.chromium
        print("🤖 : Hey, I found the chromium binary...")
        print(browser_type)
        print(
            '🤖 : Launching browser... Please wait for a while then you will see the'
            ' result.'
        )
        browser = browser_type.launch(headless=True)
        context = browser.new_context(**iphone_11)
        page = context.new_page()
        page.set_viewport_size({"width": 375, "height": 635})
        page.goto(url, wait_until="networkidle")
        # page.screenshot(path=f'./example-{browser_type.name}haha.png')
        file_name = f'screenshot{datetime.now().strftime("%m-%d_%H:%M:%S")}.png'
        page.screenshot(path=file_name)
        print(f"🤖 : DONE")
        print(f'The file is at {file_name}')
        browser.close()
    return file_name
